#!/bin/sh
#----------------------------------------------------------------------
# Description:
# Author: Norbert Flatinger
# Created at: Wed Oct 13 16:29:29 CEST 2010
#    
# Copyright (c) 2010 Norbert Flatinger  All rights reserved.
#
#----------------------------------------------------------------------
#
Version="0.1"
#
# 2011-02-18  Norbert Flatinger, IPE
#
# - this script is intended to use wget to synchronize files from a remote
#   machine directory (SRC) to a local machine directory (DEST)
# - SRC and DEST must be given as parameters
# - a semaphore file within DEST is used to prevent the script to be executed
#   more than one time (by cron)
#   in case of error, execution aborts and does not delete the semaphore; the
#   next call of the script detects the file and quits immediately; in case of
#   successful end of this script the semaphore gets deleted
#
################################################################################
#
Version="0.2"
#
# 2011-02-21  Norbert Flatinger, IPE
#
# - don't use a proxy server for file retrieval, so added option '--no-proxy';
#   using a proxy would retrieve some unwanted files generated by the proxy!
# - added some wget optins in comment
#
################################################################################
#
Version="0.3"
#
# 2011-08-26  Norbert Flatinger, IPE
#
# - set variable DEST to $2 right at the moment, when the destination directory
#   is checked for existence
#
################################################################################
#

# DEBUG=yes|no
DEBUG=yes


################################################################################
# define used commands
################################################################################
#define basename command
BASENAMEPROG="/usr/bin/basename"
# define mkdir command
MKDIRPROG="/bin/mkdir"
# define date command
DATEPROG="/bin/date"
# define rsync command
WGETPROG="/usr/bin/wget"
#RSYNCPROG="$HOME/bin/rsync.sh"
# define rm command
RMPROG="/bin/rm"


if [ -x $BASENAMEPROG ] ; then
	if [ $DEBUG = yes ] ; then
		echo -e "This is $($BASENAMEPROG "$0") version $Version\n"
	fi
else
	echo -e "\nError: cannot find or execute $BASENAMEPROG."
	echo -e "Aborting execution...\n"
	exit 1
fi

################################################################################
# Usage
################################################################################
if [ $# != 2 ] ; then
	echo -e "Usage:"
	echo -e "	$($BASENAMEPROG "$0") SRC DEST"
	echo -e " "
	echo -e "	SRC	- source"
	echo -e "	DEST	- destination"
	echo -e " "
	exit 1
fi


################################################################################
# check if destination directory exists and create it if not
################################################################################
if [ -x $MKDIRPROG ] ; then
	# destination directory
	DEST="$2"
	if [ ! -d "$DEST" ] ; then
		if [ $DEBUG = yes ] ; then
			echo -e "Destination directory $DEST does not exist."
			echo -e "Creating it ..."
		fi
		$MKDIRPROG -p "$DEST"
		RETURN_VALUE=$?
		if [ $RETURN_VALUE != 0 ] ; then
			echo -e "\nError: could not create destination directory $DEST."
			echo -e "Aborting execution ...\n"
			exit 1
		fi
	fi
else
	echo -e "\nError: cannot find or execute $MKDIRPROG."
	echo -e "Aborting execution...\n"
	exit 1
fi

# TODO: check of other arguments

# source directory
SRC="$1"


################################################################################
# check for an already running instance or previous session which ended with
# errors - in that case the semaphore file exists.
################################################################################
if [ -x $DATEPROG ] ; then
	# define semaphore
	#SEMFILE="$0".semaphore
	SEMFILE="sync.semaphore"

	# check if semaphore file exists
	if [ -e "$DEST/$SEMFILE" ] ; then
		echo -e "\nSemaphore file exists. Either due to errors of previous instance"
		echo -e "or because there is another session running in parallel."
		echo -e "Aborting execution...\n "
		exit 1
	else
		TIMESTAMP=`$DATEPROG -u +%Y-%m-%d-%H-%M-%S`
		echo "$($BASENAMEPROG "$0"): last start at $TIMESTAMP" > "$DEST/$SEMFILE"
	fi
else
	echo -e "\nError: cannot find or execute $DATEPROG."
	echo -e "Aborting execution...\n"
	exit 1
fi


################################################################################
# do synchronization with "wget"
# interesting options are:
# -b			go to background, log to 'wget-log'
# -o logfile		log to logfile
# -a logfile		append to logfile
# -q			quiet
# -N			turn on time-stamping
# --no-proxy		don't use proxies
# -nd			don't create directory hierarchy when retrieving recursively
# -nH			don't create host-prefixed directories
# -P prefix		save files to destination directory prefix
# --no-remove-listing	don't remove the temporary '.listing' file
# -r			retrieve files recursive
# -l depth		specify recursion maximum depth level
# -m			turn on options suitable for mirroring: '-r -N -l inf --no-remove-listing'
################################################################################
if [ -x $WGETPROG ] ; then
	# sync data
	$WGETPROG -m -nH -nd --no-proxy -P "$DEST" "$SRC"
	RETURN_VALUE=$?
	if [ $RETURN_VALUE != 0 ] ; then
		echo -e "\nError: rsync failed with error code $RETURN_VALUE"
		echo -e "Aborting execution ...\n"
		exit 1
	fi
else
	echo -e "\nError: cannot find or execute $RSYNCPROG."
	echo -e "Aborting execution...\n"
	exit 1
fi


################################################################################
# clean up
################################################################################
# delete semaphore file
if [ -x $RMPROG ] ; then
	$RMPROG "$DEST/$SEMFILE"
else
	echo -e "\nError: cannot find or execute $RMPROG."
	echo -e "Aborting execution...\n"
	exit 1
fi

# finish script
if [ $DEBUG = yes ] ; then
	echo "$($BASENAMEPROG "$0"): successfully finished."
fi
